@@ -3,13 +3,13 @@
  *
  *  Copyright (c) 2000 Torsten Landschoff <torsten@debian.org>
  *  		       Sebastian Krahmer  <krahmer@cs.uni-potsdam.de>
- *  
+ *
  *  License: BSD
- *  
+ *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions
  *  are met:
- *  
+ *
  *  1. Redistributions of source code must retain the above copyright
  *     notice, this list of conditions and the following disclaimer.
  *  2. Redistributions in binary form must reproduce the above copyright
@@ -19,14 +19,15 @@
  *  3. The names of the authors may not be used to endorse or promote
  *     products derived from this software without specific prior
  *     written permission.
- *  
+ *
  *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  *  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
  *  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
+
 #ifndef lint
-static const char rcsid[] =
-    "@(#) $Header: /tcpdump/master/libpcap/pcap-linux.c,v 1.73.2.6 2002/07/06 21:20:49 guy Exp $ (LBL)";
+static const char rcsid[] _U_ =
+    "@(#) $Header: /tcpdump/master/libpcap/pcap-linux.c,v 1.107 2004/03/24 06:49:16 guy Exp $ (LBL)";
 #endif
 
 /*
@@ -79,6 +80,10 @@ static const char rcsid[] =
 #include "pcap-int.h"
 #include "sll.h"
 
+#ifdef HAVE_DAG_API
+#include "pcap-dag.h"
+#endif /* HAVE_DAG_API */
+	  
 #include <errno.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -117,7 +122,7 @@ static const char rcsid[] =
  * isn't defined?  It only defines one data structure in 2.0.x, so
  * it shouldn't cause any problems.
  */
-#ifdef PF_PACKET  
+#ifdef PF_PACKET
 # include <linux/if_packet.h>
 
  /*
@@ -168,8 +173,8 @@ typedef int		socklen_t;
 
 #define MAX_LINKHEADER_SIZE	256
 
-/* 
- * When capturing on all interfaces we use this as the buffer size. 
+/*
+ * When capturing on all interfaces we use this as the buffer size.
  * Should be bigger then all MTUs that occur in real life.
  * 64kB should be enough for now.
  */
@@ -179,9 +184,14 @@ typedef int		socklen_t;
  * Prototypes for internal functions
  */
 static void map_arphrd_to_dlt(pcap_t *, int, int);
-static int live_open_old(pcap_t *, char *, int, int, char *);
-static int live_open_new(pcap_t *, char *, int, int, char *);
+static int live_open_old(pcap_t *, const char *, int, int, char *);
+static int live_open_new(pcap_t *, const char *, int, int, char *);
+static int pcap_read_linux(pcap_t *, int, pcap_handler, u_char *);
 static int pcap_read_packet(pcap_t *, pcap_handler, u_char *);
+static int pcap_inject_linux(pcap_t *, const void *, size_t);
+static int pcap_stats_linux(pcap_t *, struct pcap_stat *);
+static int pcap_setfilter_linux(pcap_t *, struct bpf_program *);
+static void pcap_close_linux(pcap_t *);
 
 /*
  * Wrap some ioctl calls
@@ -209,17 +219,18 @@ static struct sock_fprog	total_fcode
 #endif
 
 /*
- *  Get a handle for a live capture from the given device. You can 
- *  pass NULL as device to get all packages (without link level 
+ *  Get a handle for a live capture from the given device. You can
+ *  pass NULL as device to get all packages (without link level
  *  information of course). If you pass 1 as promisc the interface
- *  will be set to promiscous mode (XXX: I think this usage should 
+ *  will be set to promiscous mode (XXX: I think this usage should
  *  be deprecated and functions be added to select that later allow
  *  modification of that values -- Torsten).
- *  
+ *
  *  See also pcap(3).
  */
 pcap_t *
-pcap_open_live(char *device, int snaplen, int promisc, int to_ms, char *ebuf)
+pcap_open_live(const char *device, int snaplen, int promisc, int to_ms,
+    char *ebuf)
 {
 	pcap_t		*handle;
 	int		mtu;
@@ -227,6 +238,12 @@ pcap_open_live(char *device, int snaplen, int promisc, int to_ms, char *ebuf)
 	int		live_open_ok = 0;
 	struct utsname	utsname;
 
+#ifdef HAVE_DAG_API
+	if (strstr(device, "dag")) {
+		return dag_open_live(device, snaplen, promisc, to_ms, ebuf);
+	}
+#endif /* HAVE_DAG_API */
+
         /* Allocate a handle for this session. */
 
 	handle = malloc(sizeof(*handle));
@@ -243,7 +260,7 @@ pcap_open_live(char *device, int snaplen, int promisc, int to_ms, char *ebuf)
 	handle->md.timeout	= to_ms;
 
 	/*
-	 * NULL and "any" are special devices which give us the hint to 
+	 * NULL and "any" are special devices which give us the hint to
 	 * monitor all devices.
 	 */
 	if (!device || strcmp(device, "any") == 0) {
@@ -255,7 +272,7 @@ pcap_open_live(char *device, int snaplen, int promisc, int to_ms, char *ebuf)
 			snprintf(ebuf, PCAP_ERRBUF_SIZE,
 			    "Promiscuous mode not supported on the \"any\" device");
 		}
-	
+
 	} else
 		handle->md.device	= strdup(device);
 
@@ -266,13 +283,13 @@ pcap_open_live(char *device, int snaplen, int promisc, int to_ms, char *ebuf)
 		return NULL;
 	}
 
-	/* 
-	 * Current Linux kernels use the protocol family PF_PACKET to 
-	 * allow direct access to all packets on the network while 
-	 * older kernels had a special socket type SOCK_PACKET to 
+	/*
+	 * Current Linux kernels use the protocol family PF_PACKET to
+	 * allow direct access to all packets on the network while
+	 * older kernels had a special socket type SOCK_PACKET to
 	 * implement this feature.
 	 * While this old implementation is kind of obsolete we need
-	 * to be compatible with older kernels for a while so we are 
+	 * to be compatible with older kernels for a while so we are
 	 * trying both methods with the newer method preferred.
 	 */
 
@@ -284,10 +301,10 @@ pcap_open_live(char *device, int snaplen, int promisc, int to_ms, char *ebuf)
 			live_open_ok = 1;
 	}
 	if (!live_open_ok) {
-		/* 
+		/*
 		 * Both methods to open the packet socket failed. Tidy
 		 * up and report our failure (ebuf is expected to be
-		 * set by the functions above). 
+		 * set by the functions above).
 		 */
 
 		if (handle->md.device != NULL)
@@ -349,12 +366,7 @@ pcap_open_live(char *device, int snaplen, int promisc, int to_ms, char *ebuf)
 		 */
 		mtu = iface_get_mtu(handle->fd, device, ebuf);
 		if (mtu == -1) {
-			if (handle->md.clear_promisc)
-				/* 2.0.x kernel */
-				pcap_close_linux(handle);
-			close(handle->fd);
-			if (handle->md.device != NULL)
-				free(handle->md.device);
+			pcap_close_linux(handle);
 			free(handle);
 			return NULL;
 		}
@@ -381,26 +393,36 @@ pcap_open_live(char *device, int snaplen, int promisc, int to_ms, char *ebuf)
 	if (!handle->buffer) {
 	        snprintf(ebuf, PCAP_ERRBUF_SIZE,
 			 "malloc: %s", pcap_strerror(errno));
-		if (handle->md.clear_promisc)
-			/* 2.0.x kernel */
-			pcap_close_linux(handle);
-		close(handle->fd);
-		if (handle->md.device != NULL)
-			free(handle->md.device);
+		pcap_close_linux(handle);
 		free(handle);
 		return NULL;
 	}
 
+	/*
+	 * "handle->fd" is a socket, so "select()" and "poll()"
+	 * should work on it.
+	 */
+	handle->selectable_fd = handle->fd;
+
+	handle->read_op = pcap_read_linux;
+	handle->inject_op = pcap_inject_linux;
+	handle->setfilter_op = pcap_setfilter_linux;
+	handle->set_datalink_op = NULL;	/* can't change data link type */
+	handle->getnonblock_op = pcap_getnonblock_fd;
+	handle->setnonblock_op = pcap_setnonblock_fd;
+	handle->stats_op = pcap_stats_linux;
+	handle->close_op = pcap_close_linux;
+
 	return handle;
 }
 
 /*
  *  Read at most max_packets from the capture stream and call the callback
  *  for each of them. Returns the number of packets handled or -1 if an
- *  error occured. 
+ *  error occured.
  */
-int
-pcap_read(pcap_t *handle, int max_packets, pcap_handler callback, u_char *user)
+static int
+pcap_read_linux(pcap_t *handle, int max_packets, pcap_handler callback, u_char *user)
 {
 	/*
 	 * Currently, on Linux only one packet is delivered per read,
@@ -410,7 +432,7 @@ pcap_read(pcap_t *handle, int max_packets, pcap_handler callback, u_char *user)
 }
 
 /*
- *  Read a packet from the socket calling the handler provided by 
+ *  Read a packet from the socket calling the handler provided by
  *  the user. Returns the number of packets received or -1 if an
  *  error occured.
  */
@@ -450,10 +472,22 @@ pcap_read_packet(pcap_t *handle, pcap_handler callback, u_char *userdata)
 
 	bp = handle->buffer + handle->offset;
 	do {
+		/*
+		 * Has "pcap_breakloop()" been called?
+		 */
+		if (handle->break_loop) {
+			/*
+			 * Yes - clear the flag that indicates that it
+			 * has, and return -2 as an indication that we
+			 * were told to break out of the loop.
+			 */
+			handle->break_loop = 0;
+			return -2;
+		}
 		fromlen = sizeof(from);
-		packet_len = recvfrom( 
+		packet_len = recvfrom(
 			handle->fd, bp + offset,
-			handle->bufsize - offset, MSG_TRUNC, 
+			handle->bufsize - offset, MSG_TRUNC,
 			(struct sockaddr *) &from, &fromlen);
 	} while (packet_len == -1 && errno == EINTR);
 
@@ -544,16 +578,16 @@ pcap_read_packet(pcap_t *handle, pcap_handler callback, u_char *userdata)
 #endif
 
 	/*
-	 * XXX: According to the kernel source we should get the real 
-	 * packet len if calling recvfrom with MSG_TRUNC set. It does 
+	 * XXX: According to the kernel source we should get the real
+	 * packet len if calling recvfrom with MSG_TRUNC set. It does
 	 * not seem to work here :(, but it is supported by this code
-	 * anyway. 
+	 * anyway.
 	 * To be honest the code RELIES on that feature so this is really
 	 * broken with 2.2.x kernels.
 	 * I spend a day to figure out what's going on and I found out
-	 * that the following is happening: 
+	 * that the following is happening:
 	 *
-	 * The packet comes from a random interface and the packet_rcv 
+	 * The packet comes from a random interface and the packet_rcv
 	 * hook is called with a clone of the packet. That code inserts
 	 * the packet into the receive queue of the packet socket.
 	 * If a filter is attached to that socket that filter is run
@@ -563,10 +597,10 @@ pcap_read_packet(pcap_t *handle, pcap_handler callback, u_char *userdata)
 	 * # tcpdump -d
 	 * (000) ret      #68
 	 *
-	 * So the packet filter cuts down the packet. The recvfrom call 
+	 * So the packet filter cuts down the packet. The recvfrom call
 	 * says "hey, it's only 68 bytes, it fits into the buffer" with
-	 * the result that we don't get the real packet length. This 
-	 * is valid at least until kernel 2.2.17pre6. 
+	 * the result that we don't get the real packet length. This
+	 * is valid at least until kernel 2.2.17pre6.
 	 *
 	 * We currently handle this by making a copy of the filter
 	 * program, fixing all "ret" instructions with non-zero
@@ -640,6 +674,65 @@ pcap_read_packet(pcap_t *handle, pcap_handler callback, u_char *userdata)
 	return 1;
 }
 
+static int
+pcap_inject_linux(pcap_t *handle, const void *buf, size_t size)
+{
+#ifdef HAVE_PF_PACKET_SOCKETS
+	struct sockaddr_ll sa_ll;
+#endif
+	struct sockaddr_pkt sa_pkt;
+	int ret;
+
+#ifdef HAVE_PF_PACKET_SOCKETS
+	if (!handle->md.sock_packet) {
+		/* PF_PACKET socket */
+		if (handle->md.ifindex == -1) {
+			/*
+			 * Cooked mode - can't send.
+			 * XXX - how do you send on a bound cooked-mode
+			 * socket?
+			 */
+			strlcpy(handle->errbuf,
+			    "Sending packets isn't supported in cooked mode",
+			    PCAP_ERRBUF_SIZE);
+			return (-1);
+		}
+
+		memset(&sa_ll, 0, sizeof(sa_ll));
+		sa_ll.sll_family = AF_PACKET;
+		sa_ll.sll_ifindex = handle->md.ifindex;
+		/*
+		 * Do we have to set the hardware address?
+		 */
+		sa_ll.sll_protocol = htons(ETH_P_ALL);
+
+		ret = sendto(handle->fd, buf, size, 0, 
+			(struct sockaddr *)&sa_ll, sizeof(sa_ll));
+		if (ret == -1) {
+			snprintf(handle->errbuf, PCAP_ERRBUF_SIZE, "send: %s",
+			    pcap_strerror(errno));
+			return (-1);
+		}
+		return (ret);
+	}
+#endif
+	memset(&sa_pkt, 0, sizeof(sa_pkt));
+	sa_pkt.spkt_family = PF_INET;
+	strcpy(sa_pkt.spkt_device, handle->md.device);
+	/*
+	 * Do we have to set "spkt_protocol" to the Ethernet protocol?
+	 */
+
+	ret = sendto(handle->fd, buf, size, 0, (struct sockaddr *)&sa_pkt, 
+		sizeof(sa_pkt));
+	if (ret == -1) {
+		snprintf(handle->errbuf, PCAP_ERRBUF_SIZE, "send: %s",
+		    pcap_strerror(errno));
+		return (-1);
+	}
+	return (ret);
+}                           
+
 /*
  *  Get the statistics for the given packet capture handle.
  *  Reports the number of dropped packets iff the kernel supports
@@ -648,13 +741,15 @@ pcap_read_packet(pcap_t *handle, pcap_handler callback, u_char *userdata)
  *  patches); otherwise, that information isn't available, and we lie
  *  and report 0 as the count of dropped packets.
  */
-int
-pcap_stats(pcap_t *handle, struct pcap_stat *stats)
+static int
+pcap_stats_linux(pcap_t *handle, struct pcap_stat *stats)
 {
 #ifdef HAVE_TPACKET_STATS
 	struct tpacket_stats kstats;
 	socklen_t len = sizeof (struct tpacket_stats);
+#endif
 
+#ifdef HAVE_TPACKET_STATS
 	/*
 	 * Try to get the packet counts from the kernel.
 	 */
@@ -683,9 +778,13 @@ pcap_stats(pcap_t *handle, struct pcap_stat *stats)
 		 * platforms, but the best approximation is to return
 		 * "tp_packets" as the count of packets and "tp_drops"
 		 * as the count of drops.
+		 *
+		 * Keep a running total because each call to 
+		 *    getsockopt(handle->fd, SOL_PACKET, PACKET_STATISTICS, ....
+		 * resets the counters to zero.
 		 */
-		handle->md.stat.ps_recv = kstats.tp_packets;
-		handle->md.stat.ps_drop = kstats.tp_drops;
+		handle->md.stat.ps_recv += kstats.tp_packets;
+		handle->md.stat.ps_drop += kstats.tp_drops;
 	}
 	else
 	{
@@ -737,10 +836,29 @@ pcap_stats(pcap_t *handle, struct pcap_stat *stats)
 }
 
 /*
- *  Attach the given BPF code to the packet capture device. 
+ * Description string for the "any" device.
  */
+static const char any_descr[] = "Pseudo-device that captures on all interfaces";
+
 int
-pcap_setfilter(pcap_t *handle, struct bpf_program *filter)
+pcap_platform_finddevs(pcap_if_t **alldevsp, char *errbuf)
+{
+	if (pcap_add_if(alldevsp, "any", 0, any_descr, errbuf) < 0)
+		return (-1);
+
+#ifdef HAVE_DAG_API
+	if (dag_platform_finddevs(alldevsp, errbuf) < 0)
+		return (-1);
+#endif /* HAVE_DAG_API */
+
+	return (0);
+}
+
+/*
+ *  Attach the given BPF code to the packet capture device.
+ */
+static int
+pcap_setfilter_linux(pcap_t *handle, struct bpf_program *filter)
 {
 #ifdef SO_ATTACH_FILTER
 	struct sock_fprog	fcode;
@@ -762,26 +880,19 @@ pcap_setfilter(pcap_t *handle, struct bpf_program *filter)
 		/* install_bpf_program() filled in errbuf */
 		return -1;
 
-	/* 
-	 * Run user level packet filter by default. Will be overriden if 
-	 * installing a kernel filter succeeds. 
+	/*
+	 * Run user level packet filter by default. Will be overriden if
+	 * installing a kernel filter succeeds.
 	 */
 	handle->md.use_bpf = 0;
 
-	/*
-	 * If we're reading from a savefile, don't try to install
-	 * a kernel filter.
-	 */
-	if (handle->sf.rfile != NULL)
-		return 0;
-
 	/* Install kernel level filter if possible */
 
 #ifdef SO_ATTACH_FILTER
 #ifdef USHRT_MAX
 	if (handle->fcode.bf_len > USHRT_MAX) {
 		/*
-		 * fcode.len is an unsigned short for current kernel. 
+		 * fcode.len is an unsigned short for current kernel.
 		 * I have yet to see BPF-Code with that much
 		 * instructions but still it is possible. So for the
 		 * sake of correctness I added this check.
@@ -841,14 +952,14 @@ pcap_setfilter(pcap_t *handle, struct bpf_program *filter)
 		}
 		else if (err == -1)	/* Non-fatal error */
 		{
-			/* 
+			/*
 			 * Print a warning if we weren't able to install
 			 * the filter for a reason other than "this kernel
 			 * isn't configured to support socket filters.
 			 */
 			if (errno != ENOPROTOOPT && errno != EOPNOTSUPP) {
 				fprintf(stderr,
-				    "Warning: Kernel filter failed: %s\n", 
+				    "Warning: Kernel filter failed: %s\n",
 					pcap_strerror(errno));
 			}
 		}
@@ -880,8 +991,8 @@ pcap_setfilter(pcap_t *handle, struct bpf_program *filter)
 }
 
 /*
- *  Linux uses the ARP hardware type to identify the type of an 
- *  interface. pcap uses the DLT_xxx constants for this. This 
+ *  Linux uses the ARP hardware type to identify the type of an
+ *  interface. pcap uses the DLT_xxx constants for this. This
  *  function takes a pointer to a "pcap_t", and an ARPHRD_xxx
  *  constant, as arguments, and sets "handle->linktype" to the
  *  appropriate DLT_XXX constant and sets "handle->offset" to
@@ -894,7 +1005,7 @@ pcap_setfilter(pcap_t *handle, struct bpf_program *filter)
  *  If "cooked_ok" is non-zero, we can use DLT_LINUX_SLL and capture
  *  in cooked mode; otherwise, we can't use cooked mode, so we have
  *  to pick some type that works in raw mode, or fail.
- *  
+ *
  *  Sets the link type to -1 if unable to map the type.
  */
 static void map_arphrd_to_dlt(pcap_t *handle, int arptype, int cooked_ok)
@@ -902,6 +1013,34 @@ static void map_arphrd_to_dlt(pcap_t *handle, int arptype, int cooked_ok)
 	switch (arptype) {
 
 	case ARPHRD_ETHER:
+		/*
+		 * This is (presumably) a real Ethernet capture; give it a
+		 * link-layer-type list with DLT_EN10MB and DLT_DOCSIS, so
+		 * that an application can let you choose it, in case you're
+		 * capturing DOCSIS traffic that a Cisco Cable Modem
+		 * Termination System is putting out onto an Ethernet (it
+		 * doesn't put an Ethernet header onto the wire, it puts raw
+		 * DOCSIS frames out on the wire inside the low-level
+		 * Ethernet framing).
+		 *
+		 * XXX - are there any sorts of "fake Ethernet" that have
+		 * ARPHRD_ETHER but that *shouldn't offer DLT_DOCSIS as
+		 * a Cisco CMTS won't put traffic onto it or get traffic
+		 * bridged onto it?  ISDN is handled in "live_open_new()",
+		 * as we fall back on cooked mode there; are there any
+		 * others?
+		 */
+		handle->dlt_list = (u_int *) malloc(sizeof(u_int) * 2);
+		/*
+		 * If that fails, just leave the list empty.
+		 */
+		if (handle->dlt_list != NULL) {
+			handle->dlt_list[0] = DLT_EN10MB;
+			handle->dlt_list[1] = DLT_DOCSIS;
+			handle->dlt_count = 2;
+		}
+		/* FALLTHROUGH */
+
 	case ARPHRD_METRICOM:
 	case ARPHRD_LOOPBACK:
 		handle->linktype = DLT_EN10MB;
@@ -934,7 +1073,7 @@ static void map_arphrd_to_dlt(pcap_t *handle, int arptype, int cooked_ok)
 		break;
 
 	case ARPHRD_ARCNET:
-		handle->linktype = DLT_ARCNET;
+		handle->linktype = DLT_ARCNET_LINUX;
 		break;
 
 #ifndef ARPHRD_FDDI	/* From Linux 2.2.13 */
@@ -1048,10 +1187,10 @@ static void map_arphrd_to_dlt(pcap_t *handle, int arptype, int cooked_ok)
 		}
 		break;
 
-#ifndef ARPHRD_HDLC
-#define ARPHRD_HDLC 513	/* From Linux 2.2.13 */
+#ifndef ARPHRD_CISCO
+#define ARPHRD_CISCO 513 /* previously ARPHRD_HDLC */
 #endif
-	case ARPHRD_HDLC:
+	case ARPHRD_CISCO:
 		handle->linktype = DLT_C_HDLC;
 		break;
 
@@ -1071,6 +1210,10 @@ static void map_arphrd_to_dlt(pcap_t *handle, int arptype, int cooked_ok)
 #define ARPHRD_RAWHDLC 518
 #endif
 	case ARPHRD_RAWHDLC:
+#ifndef ARPHRD_DLCI
+#define ARPHRD_DLCI 15
+#endif
+	case ARPHRD_DLCI:
 		/*
 		 * XXX - should some of those be mapped to DLT_LINUX_SLL
 		 * instead?  Should we just map all of them to DLT_LINUX_SLL?
@@ -1078,10 +1221,49 @@ static void map_arphrd_to_dlt(pcap_t *handle, int arptype, int cooked_ok)
 		handle->linktype = DLT_RAW;
 		break;
 
+#ifndef ARPHRD_FRAD
+#define ARPHRD_FRAD 770
+#endif
+	case ARPHRD_FRAD:
+		handle->linktype = DLT_FRELAY;
+		break;
+
 	case ARPHRD_LOCALTLK:
 		handle->linktype = DLT_LTALK;
 		break;
 
+#ifndef ARPHRD_FCPP
+#define ARPHRD_FCPP	784
+#endif
+	case ARPHRD_FCPP:
+#ifndef ARPHRD_FCAL
+#define ARPHRD_FCAL	785
+#endif
+	case ARPHRD_FCAL:
+#ifndef ARPHRD_FCPL
+#define ARPHRD_FCPL	786
+#endif
+	case ARPHRD_FCPL:
+#ifndef ARPHRD_FCFABRIC
+#define ARPHRD_FCFABRIC	787
+#endif
+	case ARPHRD_FCFABRIC:
+		/*
+		 * We assume that those all mean RFC 2625 IP-over-
+		 * Fibre Channel, with the RFC 2625 header at
+		 * the beginning of the packet.
+		 */
+		handle->linktype = DLT_IP_OVER_FC;
+		break;
+
+	case ARPHRD_IRDA:
+		/* Don't expect IP packet out of this interfaces... */
+		handle->linktype = DLT_LINUX_IRDA;
+		/* We need to save packet direction for IrDA decoding,
+		 * so let's use "Linux-cooked" mode. Jean II */
+		//handle->md.cooked = 1;
+		break;
+
 	default:
 		handle->linktype = -1;
 		break;
@@ -1096,11 +1278,11 @@ static void map_arphrd_to_dlt(pcap_t *handle, int arptype, int cooked_ok)
  *  FIXME: 0 uses to mean success (Sebastian)
  */
 static int
-live_open_new(pcap_t *handle, char *device, int promisc, 
+live_open_new(pcap_t *handle, const char *device, int promisc,
 	      int to_ms, char *ebuf)
 {
 #ifdef HAVE_PF_PACKET_SOCKETS
-	int			sock_fd = -1, device_id, arptype;
+	int			sock_fd = -1, arptype;
 	int			err;
 	int			fatal_err = 0;
 	struct packet_mreq	mr;
@@ -1110,10 +1292,10 @@ live_open_new(pcap_t *handle, char *device, int promisc,
 	do {
 		/*
 		 * Open a socket with protocol family packet. If a device is
-		 * given we try to open it in raw mode otherwise we use 
-		 * the cooked interface. 
+		 * given we try to open it in raw mode otherwise we use
+		 * the cooked interface.
 		 */
-		sock_fd = device ? 
+		sock_fd = device ?
 			socket(PF_PACKET, SOCK_RAW, htons(ETH_P_ALL))
 		      : socket(PF_PACKET, SOCK_DGRAM, htons(ETH_P_ALL));
 
@@ -1146,8 +1328,8 @@ live_open_new(pcap_t *handle, char *device, int promisc,
 		handle->offset	 = 0;
 
 		/*
-		 * What kind of frames do we have to deal with? Fall back 
-		 * to cooked mode if we have an unknown interface type. 
+		 * What kind of frames do we have to deal with? Fall back
+		 * to cooked mode if we have an unknown interface type.
 		 */
 
 		if (device) {
@@ -1162,6 +1344,7 @@ live_open_new(pcap_t *handle, char *device, int promisc,
 			map_arphrd_to_dlt(handle, arptype, 1);
 			if (handle->linktype == -1 ||
 			    handle->linktype == DLT_LINUX_SLL ||
+			    handle->linktype == DLT_LINUX_IRDA ||
 			    (handle->linktype == DLT_EN10MB &&
 			     (strncmp("isdn", device, 4) == 0 ||
 			      strncmp("isdY", device, 4) == 0))) {
@@ -1179,7 +1362,7 @@ live_open_new(pcap_t *handle, char *device, int promisc,
 						 "close: %s", pcap_strerror(errno));
 					break;
 				}
-				sock_fd = socket(PF_PACKET, SOCK_DGRAM, 
+				sock_fd = socket(PF_PACKET, SOCK_DGRAM,
 						 htons(ETH_P_ALL));
 				if (sock_fd == -1) {
 					snprintf(ebuf, PCAP_ERRBUF_SIZE,
@@ -1188,6 +1371,17 @@ live_open_new(pcap_t *handle, char *device, int promisc,
 				}
 				handle->md.cooked = 1;
 
+				/*
+				 * Get rid of any link-layer type list
+				 * we allocated - this only supports cooked
+				 * capture.
+				 */
+				if (handle->dlt_list != NULL) {
+					free(handle->dlt_list);
+					handle->dlt_list = NULL;
+					handle->dlt_count = 0;
+				}
+
 				if (handle->linktype == -1) {
 					/*
 					 * Warn that we're falling back on
@@ -1202,14 +1396,18 @@ live_open_new(pcap_t *handle, char *device, int promisc,
 						"socket",
 						arptype);
 				}
-				handle->linktype = DLT_LINUX_SLL;
+				/* IrDA capture is not a real "cooked" capture,
+				 * it's IrLAP frames, not IP packets. */
+				if (handle->linktype != DLT_LINUX_IRDA)
+					handle->linktype = DLT_LINUX_SLL;
 			}
 
-			device_id = iface_get_id(sock_fd, device, ebuf);
-			if (device_id == -1)
+			handle->md.ifindex = iface_get_id(sock_fd, device, ebuf);
+			if (handle->md.ifindex == -1)
 				break;
 
-			if ((err = iface_bind(sock_fd, device_id, ebuf)) < 0) {
+			if ((err = iface_bind(sock_fd, handle->md.ifindex,
+			    ebuf)) < 0) {
 				if (err == -2)
 					fatal_err = 1;
 				break;
@@ -1222,32 +1420,44 @@ live_open_new(pcap_t *handle, char *device, int promisc,
 			handle->linktype = DLT_LINUX_SLL;
 
 			/*
-			 * XXX - squelch GCC complaints about
-			 * uninitialized variables; if we can't
-			 * select promiscuous mode on all interfaces,
-			 * we should move the code below into the
-			 * "if (device)" branch of the "if" and
-			 * get rid of the next statement.
+			 * We're not bound to a device.
+			 * XXX - true?  Or true only if we're using
+			 * the "any" device?
+			 * For now, we're using this as an indication
+			 * that we can't transmit; stop doing that only
+			 * if we figure out how to transmit in cooked
+			 * mode.
 			 */
-			device_id = -1;
+			handle->md.ifindex = -1;
 		}
 
-		/* Select promiscuous mode on/off */
+		/*
+		 * Select promiscuous mode on if "promisc" is set.
+		 *
+		 * Do not turn allmulti mode on if we don't select
+		 * promiscuous mode - on some devices (e.g., Orinoco
+		 * wireless interfaces), allmulti mode isn't supported
+		 * and the driver implements it by turning promiscuous
+		 * mode on, and that screws up the operation of the
+		 * card as a normal networking interface, and on no
+		 * other platform I know of does starting a non-
+		 * promiscuous capture affect which multicast packets
+		 * are received by the interface.
+		 */
 
-		/* 
+		/*
 		 * Hmm, how can we set promiscuous mode on all interfaces?
 		 * I am not sure if that is possible at all.
 		 */
 
-		if (device) {
+		if (device && promisc) {
 			memset(&mr, 0, sizeof(mr));
-			mr.mr_ifindex = device_id;
-			mr.mr_type    = promisc ? 
-				PACKET_MR_PROMISC : PACKET_MR_ALLMULTI;
-			if (setsockopt(sock_fd, SOL_PACKET, 
+			mr.mr_ifindex = handle->md.ifindex;
+			mr.mr_type    = PACKET_MR_PROMISC;
+			if (setsockopt(sock_fd, SOL_PACKET,
 				PACKET_ADD_MEMBERSHIP, &mr, sizeof(mr)) == -1)
 			{
-				snprintf(ebuf, PCAP_ERRBUF_SIZE, 
+				snprintf(ebuf, PCAP_ERRBUF_SIZE,
 					"setsockopt: %s", pcap_strerror(errno));
 				break;
 			}
@@ -1264,13 +1474,18 @@ live_open_new(pcap_t *handle, char *device, int promisc,
 	if (sock_fd != -1)
 		close(sock_fd);
 
-	if (fatal_err)
+	if (fatal_err) {
+		/*
+		 * Get rid of any link-layer type list we allocated.
+		 */
+		if (handle->dlt_list != NULL)
+			free(handle->dlt_list);
 		return -2;
-	else
+	} else
 		return 0;
 #else
-	strncpy(ebuf, 
-		"New packet capturing interface not supported by build " 
+	strncpy(ebuf,
+		"New packet capturing interface not supported by build "
 		"environment", PCAP_ERRBUF_SIZE);
 	return 0;
 #endif
@@ -1278,7 +1493,7 @@ live_open_new(pcap_t *handle, char *device, int promisc,
 
 #ifdef HAVE_PF_PACKET_SOCKETS
 /*
- *  Return the index of the given device name. Fill ebuf and return 
+ *  Return the index of the given device name. Fill ebuf and return
  *  -1 on failure.
  */
 static int
@@ -1299,7 +1514,7 @@ iface_get_id(int fd, const char *device, char *ebuf)
 }
 
 /*
- *  Bind the socket associated with FD to the given device. 
+ *  Bind the socket associated with FD to the given device.
  */
 static int
 iface_bind(int fd, int ifindex, char *ebuf)
@@ -1327,9 +1542,9 @@ iface_bind(int fd, int ifindex, char *ebuf)
 		return -2;
 	}
 
-	if (err > 0) { 
+	if (err > 0) {
 		snprintf(ebuf, PCAP_ERRBUF_SIZE,
-			"bind: %s", pcap_strerror(err)); 
+			"bind: %s", pcap_strerror(err));
 		return -2;
 	}
 
@@ -1344,7 +1559,7 @@ iface_bind(int fd, int ifindex, char *ebuf)
 /*
  * With older kernels promiscuous mode is kind of interesting because we
  * have to reset the interface before exiting. The problem can't really
- * be solved without some daemon taking care of managing usage counts. 
+ * be solved without some daemon taking care of managing usage counts.
  * If we put the interface into promiscuous mode, we set a flag indicating
  * that we must take it out of that mode when the interface is closed,
  * and, when closing the interface, if that flag is set we take it out
@@ -1373,7 +1588,7 @@ static void	pcap_close_all(void)
 		pcap_close(handle);
 }
 
-void	pcap_close_linux( pcap_t *handle )
+static void	pcap_close_linux( pcap_t *handle )
 {
 	struct pcap	*p, *prevp;
 	struct ifreq	ifr;
@@ -1391,7 +1606,7 @@ void	pcap_close_linux( pcap_t *handle )
 		memset(&ifr, 0, sizeof(ifr));
 		strncpy(ifr.ifr_name, handle->md.device, sizeof(ifr.ifr_name));
 		if (ioctl(handle->fd, SIOCGIFFLAGS, &ifr) == -1) {
-			fprintf(stderr, 
+			fprintf(stderr,
 			    "Can't restore interface flags (SIOCGIFFLAGS failed: %s).\n"
 			    "Please adjust manually.\n"
 			    "Hint: This can't happen with Linux >= 2.2.0.\n",
@@ -1404,7 +1619,7 @@ void	pcap_close_linux( pcap_t *handle )
 				 */
 				ifr.ifr_flags &= ~IFF_PROMISC;
 				if (ioctl(handle->fd, SIOCSIFFLAGS, &ifr) == -1) {
-					fprintf(stderr, 
+					fprintf(stderr,
 					    "Can't restore interface flags (SIOCSIFFLAGS failed: %s).\n"
 					    "Please adjust manually.\n"
 					    "Hint: This can't happen with Linux >= 2.2.0.\n",
@@ -1441,7 +1656,11 @@ void	pcap_close_linux( pcap_t *handle )
 
 	if (handle->md.device != NULL)
 		free(handle->md.device);
-		handle->md.device = NULL;
+	handle->md.device = NULL;
+	if (handle->buffer != NULL)
+		free(handle->buffer);
+	if (handle->fd >= 0)
+		close(handle->fd);
 }
 
 /*
@@ -1450,17 +1669,17 @@ void	pcap_close_linux( pcap_t *handle )
  *  FIXME: 0 uses to mean success (Sebastian)
  */
 static int
-live_open_old(pcap_t *handle, char *device, int promisc, 
+live_open_old(pcap_t *handle, const char *device, int promisc,
 	      int to_ms, char *ebuf)
 {
-	int		sock_fd = -1, arptype;
+	int		arptype;
 	struct ifreq	ifr;
 
 	do {
 		/* Open the socket */
 
-		sock_fd = socket(PF_INET, SOCK_PACKET, htons(ETH_P_ALL));
-		if (sock_fd == -1) {
+		handle->fd = socket(PF_INET, SOCK_PACKET, htons(ETH_P_ALL));
+		if (handle->fd == -1) {
 			snprintf(ebuf, PCAP_ERRBUF_SIZE,
 				 "socket: %s", pcap_strerror(errno));
 			break;
@@ -1479,13 +1698,13 @@ live_open_old(pcap_t *handle, char *device, int promisc,
 				PCAP_ERRBUF_SIZE);
 			break;
 		}
-		if (iface_bind_old(sock_fd, device, ebuf) == -1)
+		if (iface_bind_old(handle->fd, device, ebuf) == -1)
 			break;
 
 		/*
 		 * Try to get the link-layer type.
 		 */
-		arptype = iface_get_arptype(sock_fd, device, ebuf);
+		arptype = iface_get_arptype(handle->fd, device, ebuf);
 		if (arptype == -1)
 			break;
 
@@ -1505,7 +1724,7 @@ live_open_old(pcap_t *handle, char *device, int promisc,
 		if (promisc) {
 			memset(&ifr, 0, sizeof(ifr));
 			strncpy(ifr.ifr_name, device, sizeof(ifr.ifr_name));
-			if (ioctl(sock_fd, SIOCGIFFLAGS, &ifr) == -1) {
+			if (ioctl(handle->fd, SIOCGIFFLAGS, &ifr) == -1) {
 				snprintf(ebuf, PCAP_ERRBUF_SIZE,
 					 "ioctl: %s", pcap_strerror(errno));
 				break;
@@ -1539,7 +1758,7 @@ live_open_old(pcap_t *handle, char *device, int promisc,
 				}
 
 				ifr.ifr_flags |= IFF_PROMISC;
-				if (ioctl(sock_fd, SIOCSIFFLAGS, &ifr) == -1) {
+				if (ioctl(handle->fd, SIOCSIFFLAGS, &ifr) == -1) {
 				        snprintf(ebuf, PCAP_ERRBUF_SIZE,
 						 "ioctl: %s",
 						 pcap_strerror(errno));
@@ -1556,10 +1775,6 @@ live_open_old(pcap_t *handle, char *device, int promisc,
 			}
 		}
 
-		/* Save the socket FD in the pcap structure */
-
-		handle->fd 	 = sock_fd;
-
 		/*
 		 * Default value for offset to align link-layer payload
 		 * on a 4-byte boundary.
@@ -1570,15 +1785,12 @@ live_open_old(pcap_t *handle, char *device, int promisc,
 
 	} while (0);
 
-	if (handle->md.clear_promisc)
-		pcap_close_linux(handle);
-	if (sock_fd != -1)
-		close(sock_fd);
+	pcap_close_linux(handle);
 	return 0;
 }
 
 /*
- *  Bind the socket associated with FD to the given device using the 
+ *  Bind the socket associated with FD to the given device using the
  *  interface of the old kernels.
  */
 static int
@@ -1604,9 +1816,9 @@ iface_bind_old(int fd, const char *device, char *ebuf)
 		return -1;
 	}
 
-	if (err > 0) { 
+	if (err > 0) {
 		snprintf(ebuf, PCAP_ERRBUF_SIZE,
-			"bind: %s", pcap_strerror(err)); 
+			"bind: %s", pcap_strerror(err));
 		return -1;
 	}
 
@@ -1617,7 +1829,7 @@ iface_bind_old(int fd, const char *device, char *ebuf)
 /* ===== System calls available on all supported kernels ============== */
 
 /*
- *  Query the kernel for the MTU of the given interface. 
+ *  Query the kernel for the MTU of the given interface.
  */
 static int
 iface_get_mtu(int fd, const char *device, char *ebuf)
@@ -1823,7 +2035,7 @@ set_kernel_filter(pcap_t *handle, struct sock_fprog *fcode)
 	 * the filtering done in userland even if it could have been
 	 * done in the kernel.
 	 */
-	if (setsockopt(handle->fd, SOL_SOCKET, SO_ATTACH_FILTER, 
+	if (setsockopt(handle->fd, SOL_SOCKET, SO_ATTACH_FILTER,
 		       &total_fcode, sizeof(total_fcode)) == 0) {
 		char drain[1];
 
@@ -1859,7 +2071,7 @@ set_kernel_filter(pcap_t *handle, struct sock_fprog *fcode)
 	/*
 	 * Now attach the new filter.
 	 */
-	ret = setsockopt(handle->fd, SOL_SOCKET, SO_ATTACH_FILTER, 
+	ret = setsockopt(handle->fd, SOL_SOCKET, SO_ATTACH_FILTER,
 			 fcode, sizeof(*fcode));
 	if (ret == -1 && total_filter_on) {
 		/*
@@ -1883,7 +2095,7 @@ set_kernel_filter(pcap_t *handle, struct sock_fprog *fcode)
 
 		errno = save_errno;
 	}
-	return ret;	 
+	return ret;
 }
 
 static int
